"""This is a sample module used for testing doctest.

This module is for testing how doctest handles a module with docstrings
but no doctest examples.

"""


class Foo(object):
    """A docstring with no doctest examples.

    """

    def __init__(self):
        pass
